
#include <iostream>
#include <set>
#include <string>

#include "itree.hh"

using std::cout;
using std::endl;

using intset = std::set<int>;

std::string
stringify(intset &s, intset::iterator i)
{
  if (i == s.end())
    return "END";
  return std::to_string(*i);
}

void
stl_bounds()
{
  intset s {1,3,5,7,9};
#define PRINT(exp) \
  cout << "s." #exp << " -> " << stringify(s, s.exp) << endl
  PRINT(lower_bound(2));
  PRINT(lower_bound(3));
  PRINT(upper_bound(7));
  PRINT(upper_bound(8));
  PRINT(upper_bound(9));
#undef PRINT
}

struct elem {
  const int val;
  itree_entry link;
  elem(int v) : val(v) {}
};

using inttree = itree<&elem::val, &elem::link>;

std::string
stringify(elem *ep)
{
  if (!ep)
    return "NULL";
  return std::to_string(ep->val);
}

// Print ints in range [low,high)
// I.e., ints i such that low <= i < high
void
print_between(inttree &t, int low, int high)
{
  elem *e = t.lower_bound(low);
  elem *end = t.lower_bound(high);
  for (; e != end; e = inttree::next(e))
    cout << "  " << e->val << endl;
}

void
itree_bounds()
{
  inttree t;
  for (int i : {1,3,5,7,9})
    t.insert(new elem(i));
#define PRINT(exp) \
  cout << "t." #exp << " -> " << stringify(t.exp) << endl
  PRINT(lower_bound(2));
  PRINT(lower_bound(3));
  PRINT(upper_bound(7));
  PRINT(upper_bound(8));
  PRINT(upper_bound(9));
#undef PRINT

  cout << "Printing integers in [2,7):" << endl;
  print_between(t, 2, 7);

  t.delete_all();
}

int
main()
{
  stl_bounds();
  //itree_bounds();
}
