
#include <stdlib.h>

#include <functional>
#include <iostream>
#include <exception>
#include <stdexcept>

struct my_exception : std::exception {
  const char *what() const noexcept override {
    return "This is my exception.";
  }
};

bool
doit(std::function<void()> fn)
{
  try {
    fn();
    return true;
  }
  catch (const my_exception &e) {
    std::cerr << "Doing something special for my_exception" << std::endl;
    return true;
  }
  catch (const std::exception &e) {
    std::cerr << "generic exception: " << e.what() << std::endl;
  }
  catch (const int &i) {
    std::cerr << "You threw int " << i << ", but why?" << std::endl;
  }
  catch (...) {
    std::cerr << "unknown exception type" << std::endl;
  }
  return false;
}

int
main()
{
#define TRY(e)								\
  do {									\
    std::cout << "Trying: " << #e << std::endl;				\
    bool res = doit([](){ e; });					\
    std::cout << ">>> " << (res ? "OK\n" : "error\n") << std::endl;	\
  } while(0)

  TRY(throw std::runtime_error("I'm in a bad mood"));
  TRY(throw 111);
  TRY(throw my_exception{});
  TRY(throw "Hello?");
#undef TRY

  std::cout << "Program is finished" << std::endl;
}
